import UnknownInstructionError from "../errors/UnknownInstructionError";
import InputReader from "../inputReader";
import { Cells } from "../registers";
import executeAdd from "./addInstruction";
import executeHalt from "./executeHalt";
import executeJump from "./executeJump";
import executeJumpIfEqual from "./executeJumpIfEqual";
import executeJumpIfLess from "./executeJumpIfLess";
import executeCopy from "./executeMemCopy";
import executeRead from "./executeRead";
import executePut from "./putInstruction";
import executeSubtract from "./subtractionInstruction";

export const executeStep = (
  cells: Cells, 
  programCounter: number, 
  inputReader: InputReader
) => {
  return {
    previousCells: cells.copy(),
    previousPC: programCounter,
    ...executeInstruction(cells, programCounter, inputReader)
  };
};

export const getInstructionLength = (cells: Cells, programCounter: number) => {
  const instructionCode = cells.loadValueFrom(programCounter);
  switch (instructionCode) {
    case 0: return 3;
    case 1: return 4;
    case 2: return 4;
    case 3: return 2;
    case 4: return 4;
    case 5: return 4;
    case 6: return 2;
    case 9: return 1;
    case 7: return 3;
    default: return 0;
  }
};

export const executeInstruction = (
  cells: Cells,
  programCounter: number,
  inputReader: InputReader
) => {
  const instructionCode = cells.loadValueFrom(programCounter);
  switch (instructionCode) {
    case 0: return executePut(cells, programCounter);
    case 1: return executeAdd(cells, programCounter);
    case 2: return executeSubtract(cells, programCounter);
    case 3: return executeJump(cells, programCounter);
    case 4: return executeJumpIfEqual(cells, programCounter);
    case 5: return executeJumpIfLess(cells, programCounter);
    case 6: return executeRead(cells, programCounter, inputReader);
    case 7: return executeCopy(cells, programCounter);
    case 9: return executeHalt(cells, programCounter);
    default: throw new UnknownInstructionError(instructionCode);
  }
};
