import CellDoesNotExistsError from "./errors/cellDoesNotExistsError";
import EmptyCellError from "./errors/emptyCellError";

export type Register  = {
  value: number| undefined;
  id: number;
}

export class Cells {
  maxSize: number;
  cells: Register[];
  constructor(maxSize: number, registers: Register[]) {
    this.maxSize = maxSize;
    this.cells = registers;
  }
  static getInstance(maxSize: number, registers: number[]) {
    return new Cells(
      maxSize,
      registers.map((value, index) => ({ value, id: index + 1 }))
    );
  }

  loadValueFrom(cellNumber: number) {
    this.validateCellIsPresent(cellNumber);
    const value = (this.cells[cellNumber - 1] || {}).value;
    if (value !== undefined) {
      return value;
    }
    else {
      throw new EmptyCellError(cellNumber);
    }
  }
  putValueIn(cellNumber: number, value: number) {
    this.validateCellIsPresent(cellNumber);
    this.cells[cellNumber - 1].value = value;
  }
  validateCellIsPresent(cellNumber: number) {
    if (isNaN(cellNumber) || cellNumber < 1 || cellNumber > this.cells.length) {
      throw new CellDoesNotExistsError(cellNumber);
    }
  }
  copy() {
    return new Cells(this.maxSize, this.cells.map(r => ({ ...r })));
  }
}
exports.default = Cells;
